/***************************************************************************
 * Client.java
 **************************************************************************/
package kfmes.jremote.client;

import java.net.Socket;
import java.util.HashMap;

import kfmes.jremote.common.EventListener;
import kfmes.jremote.common.ImageData;
import kfmes.jremote.common.Message;
import kfmes.jremote.common.KeyEvent;
import kfmes.jremote.common.MouseEvent;
import kfmes.jremote.common.SystemEvent;
import kfmes.jremote.net.NetworkHandler;

/**
 * @author kfmes
 * @version $Id: Client.java,v 1.1 2008/08/24 09:43:32 kfmes Exp $
 */
public class Client implements EventListener{

	private ClientUI ui = null;
	private NetworkHandler network = null;
	
	private String destAddr = "localhost";
	private int destPort = 8001;
	private boolean waitFlag; 

	public static boolean isTesting = false;
	
	private boolean isStarted = false;
	
	
	public Client() {
		init();	
	}
	
	
	public Client(String destAddr, int destPort){
		this.destAddr = destAddr;
		this.destPort = destPort;
		init();
	}
	
	public String getDestAddr() {
		return destAddr;
	}
	public int getDestPort() {
		return destPort;
	}
	
	
	public void setDestAddr(String destAddr) {
		this.destAddr = destAddr;
	}

	public void setDestPort(int destPort) {
		this.destPort = destPort;
	}
	
	protected void setNetworkHandler(NetworkHandler network){
		this.network = network;		
	}

	
	
	private void init(){
		ui = new ClientUI();
		ui.setProcess(this);	
		waitFlag = true;
		
		if(isTesting) return;
			
		try{
			while(waitFlag){
				Thread.sleep(1000);
			}
		}catch (Exception e) {
		}
		System.out.println("new socket");
		try{
			NetworkHandler handle = new NetworkHandler(new Socket(destAddr, destPort));
			setNetworkHandler(handle);
			network.setListener(this);
			
		}catch (Exception e) {
			e.printStackTrace();
		}
		
		System.out.println("before run");
		run();
		
		
	}
	
	protected void release(){
		this.waitFlag = false;
	}
	
	public void terminate(){
		System.out.println("end");
		network.terminate();
	}
	
	
//	public void setSocket(Socket socket){
//		network = new NetworkHandler(socket);
//		network.setListener(this);
//	}
//	
	public void run() {
		if(network==null)
			return;
		
		System.out.println("client.run");
		sendConnect();
		if(isTesting) return;
		network.start();
	
	}
	
	/**
	 * 접속 신호를 보낸다 
	 * 나중에 인증 처리를 이곳에서 다뤄야 할듯.
	 */
	private void sendConnect(){
		SystemEvent e = 
			new SystemEvent(SystemEvent.PeerConnect, null, Message.FROM_CLIENT);
		sendMessage(e);
	}
	
	
	public void ImageReceived(ImageData img) {
		System.out.println("Client.ImageReceived()");
		ui.processImage(img);
		
	}

	
	public void KeyEvent(KeyEvent e) {
		
	}

	
	public void MouseEvent(MouseEvent e) {
		
	}

	
	
	public void sendMessage(Message message){
		network.sendObject(message);		
	}

	public static void main(String[] args){
		new Client();
	}


	public void SystemEvent(SystemEvent message) {
		
		Object mo = message.getTag();
		HashMap<String, String> m =null ;
		if(mo!=null)
			m = (HashMap<String, String>)mo; 
		
		
		switch ((Integer)message.getContent()) {
			case SystemEvent.PeerConnect: // 이 메세지가 왔을경우는 성공적으로 연결되었을때이다. 
				isStarted = true;
				System.out.println("Client.SystemEvent().PeerConnect");
				break;
			case SystemEvent.ClientSetup: 
				System.out.println("Client.SystemEvent()");
				int width = Integer.parseInt(m.get("width"));
				int height = Integer.parseInt(m.get("height"));
				int col = Integer.parseInt(m.get("col"));
				int row = Integer.parseInt(m.get("row"));
				System.out.println("Client.SystemEvent().ClientSetup");
				ui.setScreenSize(width, height ,col, row);
			
			break;
		

		default:
			break;
		}

	}


}
