package kfmes.jremote.server;

import java.awt.AWTException;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Array;
import java.net.Socket;
import java.util.HashMap;

import javax.imageio.ImageIO;

import kfmes.jremote.common.EventListener;
import kfmes.jremote.common.ImageData;
import kfmes.jremote.common.KeyEvent;
import kfmes.jremote.common.Message;
import kfmes.jremote.common.MouseEvent;
import kfmes.jremote.common.SystemEvent;
import kfmes.jremote.net.NetworkHandler;

public class ServerProcess extends Thread implements EventListener{
	private Robot bot = null;
	/** Server running ... 	 */
	private boolean isRunning = true;
	
	private int blockSize = 64;
	protected int blockX = 0;
	protected int blockY = 0;
	
	private BufferedImage screen = null;
	protected Rectangle rect;
	private int pixels[] = null;
	private ImageData screenData = null; 
	
	
	private ImageData imgBlock[][] = null; // 현재 화면
	private ImageData imgBlockPrev[][] = null; // 이전 화면
	private boolean imgModify[][] = null; // 전 화면과 바뀌었는가?
	private int imgModifyCount = 0;
	
	
	private Toolkit toolkit = null;
	private Dimension d = null;
	private NetworkHandler network = null;
	private ServerUI ui = null;
	
	private boolean flag = true;
	
	private void dumpFile(){
		try {
			FileOutputStream out = new FileOutputStream(System.currentTimeMillis() + ".jpg");
			ImageIO.write(screen, "JPEG" , out);
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	public ServerProcess(Socket socket, Server mainapp) {
		network = new NetworkHandler(socket);
		this.ui = new ServerUI(mainapp);
		
		ui.setProcess(this);
		
		
		network.setListener(this);
		network.start();		
		init();
//		start();		
		
	}

	private void init(){
		toolkit = Toolkit.getDefaultToolkit();
		d= toolkit.getScreenSize();
//		rect = new Rectangle(d);
		rect = new Rectangle( 320, 240);
		
		blockX =  rect.width/blockSize;
		blockY =  rect.height/blockSize;
		
		//TODO: 나중에 제거해야함
		ui.setScreenSize();
//		
		try {
			bot = new Robot();
		} catch (AWTException e) {

		}
		imgBlock = null;
		System.gc();
		imgBlock = new ImageData[blockY][blockX];
		imgBlockPrev = new ImageData[blockY][blockX];
		imgModify = new boolean[blockY][blockX];
		
		String imageType = "JPEG";
		
		for(int y=0;y<blockY;y++)
			for(int x=0;x<blockX;x++){
				imgBlock[y][x] = 
					new ImageData(x*blockSize,y*blockSize, x, y, ImageData.TYPE_JPEG);
				imgModify[y][x] = true;
			}
	}
	
	
	
	private void setImageBlock(){
		checkModifyConut();
		
		for(int y=0;y<blockY;y++)
			for(int x=0;x<blockX;x++)
//				if(imgBlock[y][x]!=null){
					imgBlock[y][x].setData(
						screen.getSubimage(x*blockSize, y*blockSize, 
								blockSize, blockSize)
						);
//					System.out.println("ServerProcess.setImageBlock()" + y + " " +x);
//				}
			
	}
	
	/**
	 * 이전 화면을 저장한다.
	 *//*
	private void savePrevImage(){
		for(int y=0;y<imgBlock.length;y++)
			for(int x=0;x<imgBlock[y].length;x++)
				imgBlockPrev[y][x] = imgBlock[y][x].clone();		
		
	}
	*/
	
	/**
	 * 변경되었는지 update 해준다.
	 */
	private void checkModifyConut(){
		imgModifyCount = 0 ;
		for(int y=0;y<imgBlock.length;y++)
			for(int x=0;x<imgBlock[y].length;x++){
				if(imgBlockPrev[y][x]!= null){
					imgModify[y][x] = !imgBlockPrev[y][x].equals(imgBlock[y][x]);
					
				}
				
				imgBlockPrev[y][x] = imgBlock[y][x].clone();
				imgModifyCount++;
				
			
			}
		System.out.println("modifycount " + imgModifyCount);
	
	}
	
	/**
	 * 
	 */
	private void screenCapture(){

		screen = bot.createScreenCapture(rect);
//		pixels = new int[(d.height) * (d.width)];

//		screen.getRGB(0, 0, rect.width, rect.height, pixels, 0, rect.width);
//		ImageData data = new ImageData(0,0,0,0,ImageData.TYPE_JPEG);
//		data.setData(screen);
//		savePrevImage();
		setImageBlock();
		
	}
	
	/** 
	 * 전체 화면을 보낸다.
	 */
	private void sendFullImage(){
		/*
		screen.getRGB(0, 0, rect.width, rect.height, pixels, 0, rect.width);
		ImageData data = new ImageData(0,0,0,0,ImageData.TYPE_JPEG);
		
		screenData.setData(screen);
		network.sendObject(screenData);
		*/
	}
	
	/**
	 * 화면 조각을 보낸다. 
	 * 한번에 여러 조각을 보내게 됨
	 */
	private void sendImage(){
		
		String o = flag ? new String("a") : null;
		for(int y=0;y<blockY;y++)
			for(int x=0;x<blockX;x++)
				if(imgModify[y][x]){
					ui.drawScreenTest(imgBlock[y][x]);
					
						imgBlock[y][x].setTag(o);
						if(flag)
							imgBlock[y][x].x -= 12;
					
					network.sendObject(imgBlock[y][x]);
				}
		
		flag = !flag;
	}
	

	@Override
	public void run() {
		long curtime = System.currentTimeMillis() ;
		long prevtime = curtime;
		sendFullImage();
		
		while(isRunning){
			curtime = System.currentTimeMillis();
			
			if(prevtime+3000 < curtime){
//				send
//				screenCapture();
				screen = bot.createScreenCapture(rect);
				setImageBlock();
//				checkModifyConut();
				sendImage();
//				dumpFile();
				prevtime = curtime;
				
			}
			
			
		}
	}
	
	
	@Override
	public void ImageReceived(ImageData img) {
		// 구현할 필요 없음
		
	}

	@Override
	public void KeyEvent(KeyEvent e) {
		
		
	}

	@Override
	public void MouseEvent(MouseEvent e) {
		Object o = e.getContent();
		Point p = (Point) Array.get(o, 0);
		int action =  Array.getInt(o, 1);
		
		switch (action) {
		case MouseEvent.Action_MOUSE_Pressed:
			
			break;
		case MouseEvent.Action_MOUSE_Moved:
			bot.mouseMove(p.x, p.y);
		default:
			break;
		}
		
	}



	protected void terminate() {
		isRunning = false;
		network.terminate();
		
	}
	
	public void sendMessage(Message message){
		network.sendObject(message); 
		
	}

	@SuppressWarnings("unchecked")
	@Override
	public void SystemEvent(SystemEvent message) {
		Object mo = message.getTag();		
		HashMap<String, String> m ;
		if(mo!=null)
			m = (HashMap<String, String>)mo; 
		
		
		switch ((Integer)message.getContent()) {
		case SystemEvent.PeerConnect:
			sendMessage(
					new SystemEvent(SystemEvent.PeerConnect, null, Message.FROM_SERVER));
			HashMap<String, String> mm  =new HashMap<String, String>();
			mm.put("width", String.valueOf(rect.width));
			mm.put("height", String.valueOf(rect.height));
			mm.put("col", String.valueOf(blockX));
			mm.put("row", String.valueOf(blockY));
			sendMessage(
					new SystemEvent(SystemEvent.ClientSetup, mm, Message.FROM_SERVER)
					);
			
			start();// 이미지 데이터 보내기 시작
			
			break;
		

		default:
			break;
		}

		
	}
	
	
}
